/*
 ** tcp.h
 ** 
 ** Made by (amaury bouchard)
 ** Login   <boucha_a@epita.fr>
 ** 
 ** Started on  Sat Jul 10 10:32:53 1999 amaury bouchard
 ** Last update Mon Jul 12 13:51:13 1999 amaury bouchard
 */

/*
** *************************** TCP LIB v1.0 ************************************
** 
** librairie ayant pour but de creer facilement des connexions client-serveur
** basees sur le protocole TCP-IP.
**
** bientot dans la v1.5 : gestion des exceptions
**
** ****************************************************************************
*/

#ifndef   __TCP_SERVER_H__
#define   __TCP_SERVER_H__

#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

/* definition des etats possibles pour un fd (utilise ou non) */
#define	FD_STATE_NOT_USED	0
#define	FD_STATE_USED		(1<<0)

/* definition du type de connexion tcp (client ou server) */
#define	SERVER_SOCK		1
#define	CLIENT_SOCK		2

/* macros d'erreur */
#define	TCP_ERROR_QUIT(x)	{perror(x);exit(1);}
#define	TCP_ERROR_CONTINUE(x)	{perror(x);continue;}

/*
** t_fd_table
** structure permettant de controler chaque file descriptor
** il y a une structure de ce type qui est allouee pour chaque fd gere par la
** machine
**
** state	indique si le fd est utilise (c'est-a-dire doit etre gere par
**		la fonction qui boucle sur les fd) ou non
** fonc		pointeur sur la fonction associee au fd. cette fonction sera
**		executee si le fd recoit des donnees
*/

typedef struct	s_fd_table
{
   int		state;
   void		(*fonc)();
}		t_fd_table;

/*
** t_tcp_env
** structure contenant tous les renseignements sur l'environnement TCP
**
** port			numero de port sur lequel se fait la connexion
** server_name		nom du server (mis a 0 pour un client)
** sd_connect		contient le numero de la socket par laquelle se fait
**			la connexion
** loop_condition	la boucle sur les fd se fait tant que cet entier vaut 1
**			la boucle s'arrete si il vaut 0
** max_fd_connected	contient le numero du plus grand fd sur lequel
**			il faut boucler
** fdtablesize		taille de la table des fd geree par la machine
** fd_table		tableau de t_fd_table permettant de gerer les fd
** sread		c'est un fd_set permettant d'utiliser la fonction
** data			pointeur libre pour n'importe quel type de donnee que
**			l'utilisateur pourrait vouloir retrouver
*/

typedef struct	s_tcp_env
{
   int		port;
   char		*server_name;
   int		sd_connect;
   int		*loop_condition;
   int		max_fd_connected;
   int		fdtablesize;
   t_fd_table	*fd_table;
   fd_set	sread;
   void		*data;
}		t_tcp_env;

/* Prototypes */
void		*malloc();

/*
** tcp_client_server.c
** fonctions generales servant aussi bien pour le client que pour le serveur
**
** init_tcp_env		initialise une structure t_tcp_env, qui sera utilisee
**			par la suite par les autres fonctions
** make_active_fd	permet de rendre actif un fd. utile pour associer une
**			fonction a un fd qui n'a rien a voir avec la connexion
**			tcp (ex : pour lire l'entree standard en meme temps)
** make_inactive_fd	permet de rendre inactif un fd. une reception de donnees
**			sur ce fd ne declenchera plus d'appel de fonction
** create_tcp_socket	fonction qui cree une socket bindee ou connectee
**			(suivant le type client ou serveur)
*/

t_tcp_env	*init_tcp_env(int,int,char*,void*);
void		make_active_fd(t_tcp_env*,int,void(*)());
void		make_inactive_fd(t_tcp_env*,int);
int		create_tcp_socket(int,int,char*);

/*
** tcp_connection.c
** fonctions qui lancent la boucle de lecture des fd
**
** create_tcp_server	pour un serveur, attend une connexion. cree une nouvelle
**			socket quand la connexion s'est effectuee
** create_tcp_client	pour un client, se contente d'attendre de recevoir
**			quelque chose sur un des fd
*/
void		create_tcp_server(t_tcp_env*,int*,void(*)());
void		create_tcp_client(t_tcp_env*,int*,void(*)());

#endif /* __TCP_SERVER_H__ */
