#include "trac_db.h"

// Prototypes prives
static int	_dbReadNumberSize(ClassDB*,int);
static int	_dbReadFields(ClassDB*,int);

// =============================================================================
// fonctions publiques
// =============================================================================

// dbOpen()
// fonction qui ouvre une base de donnees existante
// retourne 0 si la base n'existe pas ou n'est pas accessible, 1 sinon
// parametre :	- l'objet
int	dbOpen(ClassDB* this)
{
  int	fdFields;
  int	flag;
  char*	str1;
  char*	str2;

  flag = 0;
  // reconstitution des chemins vers les fichiers
  str1 = (char*)malloc(strlen(this->path) + strlen(FIELDS_FILE) + 2);
  strcpy(str1,this->path);
  strcat(str1,"/");
  strcat(str1,FIELDS_FILE);
  str2 = (char*)malloc(strlen(this->path) + strlen(DB_FILE) + 2);
  strcpy(str2,this->path);
  strcat(str2,"/");
  strcat(str2,DB_FILE);
  // ouverture des fichiers
  if ((fdFields = open(str1,O_RDONLY)) == -1 ||
      (this->fd = open(str2,O_RDWR)) == -1)
    flag = 1;
  free(str1);
  free(str2);
  if (flag)
    {
      this->status = DB_STATUS_BAD;
      return (0);
    }
  // lecture de la description de la base de donnees
  if (!_dbReadNumberSize(this,fdFields))
    {
      this->status = DB_STATUS_BAD;
      return (0);
    }
  if (!_dbReadFields(this,fdFields))
    {
      this->status = DB_STATUS_BAD;
      return (0);
    }
  close(fdFields);
  // calcul de la taille de la base de donnees et du nombre d'enregistrements
  this->databaseSize = lseek(this->fd,0,2);
  this->nbOfRecords = this->databaseSize / (this->recordSize + 1);
  lseek(this->fd,0,0);
  this->status = DB_STATUS_OPENED;
  return (1);
}

// =============================================================================
// fonctions privees - utilisees par les fonctions publiques ci-dessus
// =============================================================================

// _dbReadNumberSize()
// fonction qui recupere le nombre de champs et la taille totale d'un
// enregistrement
// retourne 1 si la lecture s'est bien deroulee, 0 sinon
// parametres :	- l'objet
//		- file descriptor sur le fichier de declaration des champs
static int	_dbReadNumberSize(ClassDB* this,int fdFields)
{
  char	c;
  int	nbrOfFields;
  int	fieldsSize;

  nbrOfFields = fieldsSize = 0;
  // lecture du nombre de champs
  while ((read(fdFields,&c,1)) && c != EOF && c != ':')
    nbrOfFields = (nbrOfFields * 10) + (c - '0');
  this->nbOfFields = nbrOfFields;
  if (c == EOF)
    return (0);
  // lecture de la taille totale d'un enregistrement
  while ((read(fdFields,&c,1)) && c != EOF && c != ':')
    fieldsSize = (fieldsSize * 10) + (c - '0');
  this->recordSize = fieldsSize;
  if (c == EOF)
    return (0);
  // creation des tableaux
  this->fieldsName = (char**)malloc((this->nbOfFields + 1) * sizeof(char*));
  this->fieldsName[this->nbOfFields] = 0;
  this->fieldsSize = (int*)malloc((this->nbOfFields + 1) * sizeof(int));
  this->fieldsSize[this->nbOfFields] = 0;
  return (1);
}

// _dbReadFields()
// fonction qui recupere le nom et la taille de chaque champs
// parametres :	- l'objet
//		- file descriptor sur le fichier de declaration des champs
static int	_dbReadFields(ClassDB* this,int fd)
{
  int	i;
  char	c;
  char*	name;
  int	nb;
  int	nbFields;
  
  for (i = 0,nbFields = this->nbOfFields;nbFields > 0;i++,nbFields--)
    {
      name = (char*)malloc(256);
      for (nb = 0;
	   (read(fd,&c,1)) && c != EOF && c != '=' && c != '&';
	   nb++)
	name[nb] = c;
      name[nb] = 0;
      if (c == EOF || c == '&')
	return (0);
      this->fieldsName[i] = name;
      for (nb = 0;
	   (read(fd,&c,1)) && c != EOF && c != '&';)
	nb = (nb * 10) + (c - '0');
      this->fieldsSize[i] = nb;
      if (c == EOF && nbFields)
	return (0);
    }
  return (1);
}
