#include "trac_db.h"

// Prototypes prives
static int*	_dbGetRecordNumber(ClassDB*,char*,char*);
static int*	_dbSelectWhere(ClassDB*,char**,char**);
static int	_dbGetPreviousSize(ClassDB*,int);
static void	_rmEndSpace(char*);

// =============================================================================
// fonctions publiques
// =============================================================================

// dbFindField()
// regarde si un champ existe dans la base, et retourne son numero dans
// la base, ou retourne -1 si le champ n'existe pas dans la base de donnees
// parametres :	- l'objet
//		- le nom du champ
int	dbFindField(ClassDB* this,char* field)
{
  int	i;
  int	taille;

  // recherche du champ
  for (i = taille = 0;i < this->nbOfFields;i++)
    if (!strcmp(this->fieldsName[i],field))
      return (i);
  return (-1);
}

// dbGetRecord()
// retrouve tous les enregistrements pour lesquels on connait la valeur
// d'un des champs
// retourne un tabeau contenant tous les enregistrements, et dont le dernier
// element vaut 0
// parametres :	- l'objet
//		- tableau de noms de champs (termine par 0)
//		- tableau de valeurs associees aux champs (termine par 0)
char**	dbGetRecord(ClassDB* this,char** fields,char** values)
{
  int*		recNumTab;
  int		i;
  int		offset;
  char**	resultat;

  if (this->status == DB_STATUS_BAD || this->status == DB_STATUS_CLOSED ||
      !(recNumTab = _dbSelectWhere(this,fields,values)))
    return (0);
  // creation du tableau de resultats et positionnement sur le debut de fichier
  resultat = (char**)malloc((recNumTab[0] + 1) * sizeof(char*));
  lseek(this->fd,0,0);
  // boucle sur les resultats
  for (i = 0;i < recNumTab[0];i++)
    {
      offset = (recNumTab[i+1] * (this->recordSize + 1)) + 1;
      lseek(this->fd,offset,0);
      resultat[i] = (char*)malloc(this->recordSize);
      read(this->fd,resultat[i],this->recordSize);
    }
  free(recNumTab);
  // mise a zero du dernier element et retour
  resultat[i] = 0;
  return (resultat);
}

// dbGetFieldsOfRecord()
// retourne le contenu d'un champ d'un enregistrement
// parametres :	- l'objet
//		- l'enregistrement
//		- le nom du champ
char*	dbGetFieldOfRecord(ClassDB* this,char* record,char* field)
{
  int	fieldNumber;
  int	placeInRecord;
  char*	result;

  if (this->status == DB_STATUS_BAD || this->status == DB_STATUS_CLOSED ||
      (fieldNumber = dbFindField(this,field)) == -1)
    return (0);
  placeInRecord = _dbGetPreviousSize(this,fieldNumber);
  result = (char*)malloc(this->fieldsSize[fieldNumber] + 1);
  strncpy(result,(record + placeInRecord),this->fieldsSize[fieldNumber]);
  result[this->fieldsSize[fieldNumber]] = 0;
  return (result);
}

// dbDeleteRecord()
// efface tous les enregistrements pour lesquels on connait des couples
// champ/valeur
// retourne 1 si l'operation s'est bien deroulee, 0 sinon
// parametres :	- l'objet
//		- tableau de noms de champs (termine par 0)
//		- tableau de valeurs associees aux champs (termine par 0)
int	dbDeleteRecord(ClassDB* this,char** fields,char** values)
{
  int*	recNumTab;
  int	i;
  int	offset;
  char	c;

  if (this->status == DB_STATUS_BAD || this->status == DB_STATUS_CLOSED ||
      !(recNumTab = _dbSelectWhere(this,fields,values)))
    return (0);
  c = FREE_RECORD;
  for (i = 1;i <= recNumTab[0];i++)
    {
      offset = (recNumTab[i] * (this->recordSize + 1));
      lseek(this->fd,offset,0);
      write(this->fd,&c,1);
    }
  free(recNumTab);
  return (1);
}

// dbUpdateRecord()
// met a jour tous les enregistrements pour lesquels on connait des couples
// champ/valeur
// retourne 1 si l'operation s'est bien deroulee, 0 sinon
// parametres :	- l'objet
//		- tableau de noms de champs (termine par 0)
//		- tableau de valeurs associees aux champs (termine par 0)
//		- nom du champ a mettre a jour
//		- nouvelle vaeur de ce champ
int	dbUpdateRecord(ClassDB* this,char** fields,char** values,
		       char* fieldToUpdate,char* updateValue)
{
  int*	recNumTab;
  int	fieldNumber;
  int	i;
  int	offset;

  if (this->status == DB_STATUS_BAD || this->status == DB_STATUS_CLOSED ||
      !(recNumTab = _dbSelectWhere(this,fields,values)))// ||
    {printf("pas trouve\n");return(0);}
  if ((fieldNumber = dbFindField(this,fieldToUpdate)) < 0)// ||
    {printf("champ !existe\n");exit(0);}
  if  (  strlen(updateValue) != this->fieldsSize[fieldNumber])
    {printf("taille\n");
    return (0);}
  for (i = 1;i <= recNumTab[0];i++)
    {
      offset = (recNumTab[i] * (this->recordSize + 1));
      offset += _dbGetPreviousSize(this,fieldNumber);
      offset += 1;
      lseek(this->fd,offset,0);
      write(this->fd,updateValue,this->fieldsSize[fieldNumber]);
    }
  free(recNumTab);
  return (1);
}

// dbAddRecord()
// ajoute en nouvel enregistrement a la base de donnees
// retourne 1 si l'operation s'est bien deroulee, 0 sinon
// parametres :	- l'objet
//		- l'enregistrement
int	dbAddRecord(ClassDB* this,char* record)
{
  char	c;
  char	buff;
  int	i;

  if (this->status == DB_STATUS_BAD || this->status == DB_STATUS_CLOSED ||
      strlen(record) != this->recordSize)
    return (0);
  c = USED_RECORD;
  lseek(this->fd,0,0);
  while ((i = read(this->fd,&buff,1)) && buff != EOF && buff == USED_RECORD)
    i = lseek(this->fd,this->recordSize,1);
  if (buff == EOF || i == 0)
    {
      lseek(this->fd,0,2);
      if (write(this->fd,&c,1) &&
	  write(this->fd,record,this->recordSize) == this->recordSize)
	return (1);
    }
  else if (buff == FREE_RECORD)
    {
      lseek(this->fd,-1,1);
      if (write(this->fd,&c,1) &&
	  write(this->fd,record,this->recordSize) == this->recordSize)
	return (1);
    }
  return (0);
}

// dbCreateRecordString()
// retourne la chaine de caracteres passee en parametre, en lui ajoutant
// le champ voulu ; retourne 0 en cas d'erreur
// parametres :	- l'objet
//		- la chaine de caracteres qui va contenir la requete
//		- le nom du champ
//		- la valeur du champ
char*	dbCreateRecordString(ClassDB* this,char* recordString,
			     char* field,char* value)
{
  int	taille;

  if (recordString == 0)
    recordString = (char*)malloc(this->recordSize);
  else if (strlen(recordString) != this->recordSize)
    return (0);
  if ((taille = dbFindField(this,field)) < 0)
    return (0);
  if (strlen(value) > this->fieldsSize[taille])
    return (0);
  taille = _dbGetPreviousSize(this,taille);
  strcpy(recordString+taille,value);
  return (recordString);
}

// =============================================================================
// fonctions privees - utilisees par les fonctions publiques ci-dessus
// =============================================================================

// _dbGetRecordNumer()
// retourne un tableau d'entiers contenant les numeros des enregistrements
// dont on connait la valeur d'un des champs
// le premier entier du tableau contient le nombre d'enregistrements trouves
// parametres :	- l'objet
//		- le nom du champ
//		- la valeur associee a ce champ
static int*	_dbGetRecordNumber(ClassDB* this,char* field,char* value)
{
  int		fieldNumber;
  int		placeInRecord;
  int		recordLoop;
  char*		tmpField;
  int		nbOk;
  int*		result;
  int*		tmp;
  int		i;
  char		c;

  if ((fieldNumber = dbFindField(this,field)) == -1)
    return (0);
  placeInRecord = _dbGetPreviousSize(this,fieldNumber);
  result = 0;
  // le file descriptor est place sur le debut du fichier
  lseek(this->fd,0,0);
  // boucle sur les enregistrements
  for (recordLoop = nbOk = 0;recordLoop < this->nbOfRecords;recordLoop++)
    {
      read(this->fd,&c,1);
      if (c == USED_RECORD)
	{
	  // positionne sur le champ voulu de l'enregistrement courant
	  lseek(this->fd,placeInRecord,1);
	  // recupere la valeur du champ voulu pour l'enregistrement courant
	  tmpField = (char*)malloc(this->fieldsSize[fieldNumber] + 1);
	  read(this->fd,tmpField,this->fieldsSize[fieldNumber]);
	  tmpField[this->fieldsSize[fieldNumber]] = 0;
	  _rmEndSpace(tmpField);		// enleve les espaces de fin
	  // repositionne sur l'enregistrement suivant
	  lseek(this->fd,(this->recordSize - (placeInRecord +
		this->fieldsSize[fieldNumber])),1);
	  if (!strcmp(tmpField,value))	// compare les chaines
	    {
	      nbOk++;
	      // creation du tableau
	      tmp = (int*)malloc((nbOk + 1) * sizeof(int));
	      tmp[0] = nbOk;		// enregistre le nombre d'enregistrements
	      for (i = 1;i < nbOk;i++)	// recopie la precedente version
		tmp[i] = result[i];
	      tmp[i] = recordLoop;	// enregistre le dernier record number
	      if (result)
		free(result);		// libere la precedente version
	      result = tmp;
	    }
	  free(tmpField);
	}
      else if (c == FREE_RECORD)
	lseek(this->fd,this->recordSize,1);	// to next record
      else
	  return (0);
    }
  return (result);
}

//_dbSelectWhere()
// retrouve tous les enregistrements pour lesquels on connait la valeur d'un
// des champs
// retourne un tableau contenant tous les enregistrements, et dont le dernier
// element vaut 0
// parametres :	- l'objet
//		- tableau de noms de champs (termine par 0)
//		- tableau de valeurs associees aux champs (termine par 0)
static int*	_dbSelectWhere(ClassDB* this,char** fields,char** values)
{
  int		fieldsLoop;
  int		i;
  int		j;
  int		k;
  int		nbMatch;
  int*		recNumTab;
  int*		tmp;
  int*		result;

  recNumTab = 0;
  // boucle sur les couples champ/valeur
  for (fieldsLoop = 0;fields[fieldsLoop];fieldsLoop++)
    {
      // recupere les numeros des enregistrements correspondant a ce couple
      tmp = _dbGetRecordNumber(this,fields[fieldsLoop],values[fieldsLoop]);
      if (!tmp)
	break;
      // calcul le nombre total d'enregistrements differents
      if (!recNumTab)
	{
	  nbMatch = tmp[0];
	  recNumTab = tmp;
	  continue;
	}
      else
	for (nbMatch = 0,i = 1;i <= recNumTab[0];i++)
	  for (j = 1;j <= tmp[0];j++)
	    if (recNumTab[i] == tmp[j])
	      {
		nbMatch++;
		break;
	      }
      // cree un nouveau tableau contenant les precedents sans redondance
      result = (int*)malloc((nbMatch + 1) * sizeof(int));
      result[0] = nbMatch;
      for (i = 1;i <= recNumTab[0];i++)
	  result[i] = recNumTab[i];
      for (j = 1,nbMatch = 0;j <= tmp[0];j++,i++,nbMatch = 0)
	{
	  for (k = 0;k < i;k++)
	    if (tmp[j] == recNumTab[k])
	      {
		nbMatch = 1;
		break;
	      }
	  if (!nbMatch)
	    result[i] = tmp[j];
	}
      free(recNumTab);
      free(tmp);
      recNumTab = result;
    }
  return (recNumTab);
}

// _dbGetPreviousSize()
// retourne le nombre de caracteres des champs precedent le champ dont
// on possede le numero
// parametres :	- l'objet
//		- le numero du champ
static int	_dbGetPreviousSize(ClassDB* this,int fieldNumber)
{
  int	i;
  int	result;

  for (i = result = 0;i < this->nbOfFields && i < fieldNumber;i++)
    result += this->fieldsSize[i];
  return (result);
}

// _rmEndSpace()
// place le caractere '\0' a la fin de la chaine, en enlevant les espaces
// de fin
// parametre :	- la chaine a traiter
static void	_rmEndSpace(char* str)
{
  char*	pt;

  for (pt = str;*pt;pt++)
    ;
  for (pt -= 1;*pt == ' ' || *pt == '\t';pt--)
    ;
  *(pt + 1) = 0;
}
