#include "trac_list.h"

// =============================================================================
// fonctions publiques
// =============================================================================

// listPutStart()
// ajoute un element au debut de la liste

ClassList*	listPutStart(ClassList* this, void* elem)
{
  tListElem*	newCell;

  newCell = new(tListElem);
  newCell->elem = elem;
  newCell->next = this->start;
  this->start = newCell;
  if (this->nbOfElem == 0)
    this->end = newCell;
  else
    this->start->prev = newCell;
  this->nbOfElem += 1;
  listResetIterator(this);
  return (this);
}

// listPutEnd()
// ajoute un element a la fin de la liste

ClassList*	listPutEnd(ClassList* this, void* elem)
{
  tListElem*	newCell;

  newCell = new(tListElem);
  newCell->elem = elem;
  if (this->nbOfElem == 0)
    this->start = this->end = newCell;
  else
    {
      this->end->next = newCell;
      newCell->prev = this->end;
      this->end = newCell;
    }
  this->nbOfElem += 1;
  listResetIterator(this);
  return (this);
}

// listFind()
// cherche dans la liste un element semblable a celui passe en parametre, selon
// une fonction passee elle aussi en argument
// retourne zero s'il n'y a pas d'element correspondant

void*	listFind(ClassList* this, void* elem, int (*cmp)())
{
  tListElem*	pt;

  for (pt = this->start;pt;pt = pt->next)
    if (!cmp(elem,pt->elem))
      return (pt->elem);
  return (0);
}

// listFindCell()
// idem a la fonction precedente, sauf que renvoit le tListElem contenant
// l'element, et non pas l'element lui-meme
// retourne zero s'il n'a a pas d'element correspondant

tListElem*	listFindCell(ClassList* this, void* elem, int (*cmp)())
{
  tListElem*	pt;

  for (pt = this->start;pt;pt = pt->next)
    if (!cmp(elem,pt->elem))
      return (pt);
  return (0);  
}

// listRmElem()
// elimine tous les elements de la liste qui sont semblables a celui qui est
// passe en parametre, selon la fonction de comparaison passee en argument

void	listRmElem(ClassList* this, void* elem, int (*cmp)())
{
  tListElem*	pt;
  tListElem*	pt2;

  for (pt = this->start;pt;)
    {
      if(!cmp(elem,pt->elem))
	{
	  if (pt->prev)
	    pt->prev->next = pt->next;
	  else
	    this->start = pt->next;
	  if (pt->next)
	    pt->next->prev = pt->prev;
	  else
	    this->end = pt->prev;
	  pt2 = pt;
	  pt = pt->next;
	  free(pt2);
	  this->nbOfElem -= 1;
	}
      else
	pt = pt->next;
    }
  listResetIterator(this);
}

// listRmCell()
// idem a la fonction precedente, sauf que prend en parametre le tListElem
// a detruire

void	listRmCell(ClassList* this, tListElem* cell)
{
  tListElem*	pt;
  tListElem*	pt2;

  for (pt = this->start;pt;)
    {
      if (pt == cell)
	{
	  if (pt->prev)
	    pt->prev->next = pt->next;
	  else
	    this->start = pt->next;
	  if (pt->next)
	    pt->next->prev = pt->prev;
	  else
	    this->end = pt->prev;
	  pt2 = pt;
	  pt = pt->next;
	  free(pt2);	  
	  this->nbOfElem -= 1;
	}
      else
	pt = pt->next;
    }
  listResetIterator(this);
}

// listRmNbr()
// detruit d'une liste un element dont on connait le numero dans la liste
// note : le premier element de la liste porte le numero zero

void	listRmNbr(ClassList* this, int nb)
{
  int		index;
  tListElem*	pt;

  if (nb > (this->nbOfElem - 1))
    return ;
  for (index = 0,pt = this->start;index < nb;pt = pt->next)
    ;
  listRmCell(this,pt);
}

// listSort()
// fonction de classement de liste, selon une fonction passee en parametre

void	listSort(ClassList* this, int (*cmp)())
{
  tListElem**	begin;
  tListElem*	pt;
  void*		swap;
  int		loop;
  
  for (loop = 1,begin = &this->start;loop;)
    for (loop = 0,pt = *begin;pt->next;pt = pt->next)
      if (cmp(pt->elem,pt->next->elem) > 0)
	{
	  loop++;
	  swap = pt->next->elem;
	  pt->next->elem = pt->elem;
	  pt->elem = swap;
	}
  listResetIterator(this);
}

// listResetIterator()
// fonction qui remet a zero l'iterateur de la liste

void	listResetIterator(ClassList* this)
{
  this->iterator = this->nbOfElem;
  this->iteratorPointer = this->start;
}

// listGetNextElem()
// retourne l'element suivant (grace a l'iterateur)

void*		listGetNextElem(ClassList* this)
{
  tListElem*	pt;

  if (this->iterator <= 0 || !this->iteratorPointer)
    return (0);
  this->iterator -= 1;
  pt = this->iteratorPointer;
  this->iteratorPointer = pt->next;
  return (pt->elem);
}

// listReverse()
// inverse l'ordre des lments de la liste

void		listReverse(ClassList* this)
{
  tListElem*	pt;
  tListElem*	ptNext;
  tListElem*	ptTmp;
  
  for (pt = this->start; pt; pt = ptNext)
    {
      ptNext = pt->next;
      ptTmp = pt->prev;
      pt->prev = pt->next;
      pt->next = ptTmp;
    }
  ptTmp = this->start;
  this->start = this->end;
  this->end = ptTmp;
  listResetIterator(this);
}

// =============================================================================
// constructeur / destructeur
// =============================================================================

// newClassList()
// constructeur de l'objet ClassList

ClassList*	newClassList(ClassList* this)
{
  initParentClass(this,deleteClassList);
  return (this);
}

// deleteClassList()
// destructeur de l'objet ClassList

void	deleteClassList(ClassList* this)
{
  while (this->iterator)
    listRmNbr(this,0);
}
