#include "trac_stream.h"

// streamOpenIO()
// permet d'ouvrir un stream sur un file descriptor (socket, entree standard, ...)
// si le stream precedemment gere par cet objet a ete ferme
// retourne 0 si l'operation n'a pas pu etre effectuee

ClassStream*	streamOpenIO(ClassStream* this,int fd)
{
  if (!(this->type & STREAM_OPENED))
    {
      this->type = STREAM_OPENED + STREAM_IO;
      if (fd == 0)
	this->type |= STREAM_READ;
      else if (fd == 1 || fd == 2)
	this->type |= STREAM_WRITE;
      else
	this->type |= STREAM_READ + STREAM_WRITE;
      this->fd = fd;
      return (this);
    }
  return (0);
}

// streamOpenFile()
// permet d'ouvrir un stream sur un fichier si le stream precedemment gere par
// cet objet a ete ferme
// retourne 0 si 'operation n'a pas pu etre effectuee

ClassStream*	streamOpenFile(ClassStream* this,char* fileName,int flags,
			       mode_t mode)
{
  if (!(this->type & STREAM_OPENED))
    if ((this->fd = open(fileName,flags,mode)) >= 0)
      {
	this->type = STREAM_OPENED + STREAM_FILE;
	if (mode & O_RDONLY)
	  this->type |= STREAM_READ;
	else if (mode & O_WRONLY)
	  this->type |= STREAM_WRITE;
	else if (mode & O_RDWR)
	  this->type |= STREAM_READ + STREAM_WRITE;
	this->mode = mode;
	this->flags = flags;
	this->fileSize = lseek(this->fd,0,SEEK_END);
	lseek(this->fd,0,SEEK_END);
	return (this);
      }
  return (0);
}

// streamClose()
// ferme un stream mais ne detruit pas l'objet

void	streamClose(ClassStream* this)
{
  streamFlush(this);
  close(this->fd);
  this->type = 0;
  this->charLeftRd = this->charLeftWr = 0;
  this->readPos = this->readBuff;
  this->writePos = this->writeBuff;
}

// streamFlush()
// vide les buffers

void	streamFlush(ClassStream* this)
{
  int	nbCharToWrite;

  if (this->charLeftWr)
    {
      nbCharToWrite = this->writePos - this->writeBuff + this->charLeftWr;
      write(this->fd,this->writeBuff,nbCharToWrite);
    }
}

// newClassStream()
// constructeur de ClassStream

ClassStream*	newClassStream(ClassStream* this)
{
  initParentClass(this,deleteClassStream);
  this->readBuff = (char*)malloc(STREAM_BUFF_SIZE*sizeof(char));
  this->writeBuff = (char*)malloc(STREAM_BUFF_SIZE*sizeof(char));
  return (this);
}

// deleteClassStream()
// destructeur de ClassStream

void	deleteClassStream(ClassStream* this)
{
  if (this->type & STREAM_FILE && this->type & STREAM_OPENED)
    streamCloseFile(this);
  free(this->readBuff);
  free(this->writeBuff);
}
