#include "trac_string.h"
 
// =============================================================================
// fonctions publiques
// =============================================================================                                                              

// stringConcat()
// permet de concatener une chaine de caractere a la fin de la chaine d'un
// objet ClassString

void	stringConcat(ClassString* this,const char* str)
{
  char	*newStr;
  int	size;

  size = strlen(this->str) + strlen(str);
  newStr = malloc((size+1)*sizeof(char));
  strcpy(newStr,this->str);
  strcat(newStr,str);
  free(this->str);
  this->str = newStr;
  this->nbrChar = size;
}

// stringNbrChar()
// retourne le nombre de caracteres que contient un objet ClassString

int	stringNbrChar(ClassString* this)
{
  return (this->nbrChar);
}

// stringLowerCase()
// retourne un objet ClassString dont tous les caractres sont des minuscules

ClassString*	stringLowerCase(ClassString* this)
{
  ClassString*	result;

  result = newClassString(new(ClassString), lowerCase(this->str));
  return (result);
}

// stringUpperCase()
// retourne un objet ClassString dont tous les caractres sont des majuscules

ClassString*	stringUpperCase(ClassString* this)
{
  ClassString*	result;

  result = newClassString(new(ClassString), upperCase(this->str));
  return (result);
}

// stringTrim()
// retourne un objet ClassString dont la chaine est dbarasse des espaces
// inutiles

ClassString*	stringTrim(ClassString* this)
{
  ClassString*	result;

  result = newClassString(new(ClassString), trim(this->str));
  return (result);
}

// =============================================================================
// constructeur / destructeur
// =============================================================================

// newClassString()
// constructeur de l'objet ClassString

ClassString*	newClassString(ClassString* this,const char* str)
{
  int		size;

  // initialisation de l'heritage
  initParentClass(this,deleteClassString);
  // initialisation de l'objet
  size = strlen(str);
  this->str = (char*)malloc((size+1)*sizeof(char));
  strcpy(this->str,str);
  this->nbrChar = size;
  return (this);
}

// deleteClassString()
// destructeur de l'objet ClassString

void	deleteClassString(ClassString* this)
{
  if (this->str)
    free(this->str);
}

// =============================================================================
// fonctions supplmentaires n'utilisant pas l'objet String, mais portant sur
// les chanes de caractres
// =============================================================================

// lowerCase()
// retourne une copie d'une chaine de caractres, avec toutes les lettres en
// minuscule

char*	lowerCase(const char* str)
{
  char*	result;
  int	strSize;
  int	i;
  
  if (!str || !(strSize = strlen(str)))
    return (NULL);
  result = (char*)malloc(strSize + 1);
  for (i = 0; i <= strSize; i++)
    if (str[i] >= 'A' && str[i] <= 'Z')
      result[i] = 'a' + (str[i] - 'A');
    else
      result[i] = str[i];
  return (result);
}

// upperCase()
// retourne une copie d'une chaine de caractres avec toutes les lettres en
// majuscule

char*	upperCase(const char* str)
{
  char*	result;
  int	strSize;
  int	i;

  if (!str || !(strSize = strlen(str)))
    return (NULL);
  result = (char*)malloc(strSize + 1);
  for (i = 0; i <= strSize; i++)
    if (str[i] >= 'a' && str[i] <= 'z')
      result[i] = 'A' + (str[i] - 'a');
    else
      result[i] = str[i];
  return (result);
}

// trim()
// retourne une copie d'une chaine de caractres dbarasse des espaces au
// dbut et  la fin de la chaine

char*	trim(const char* str)
{
  char*	result;
  int	i;
  int	beginStr;
  int	strSize;

  if (!str || !(strSize = strlen(str)))
    return (NULL);
  for (i = 0; str[i] == ' ' || str[i] == '\t' || str[i] == '\n'; i++)
    ;
  beginStr = i;
  i = strSize - 1;
  while ((str[i] == ' ' || str[i] == '\t' || str[i] == '\n' || str[i] < 9) &&
	 i >= beginStr)
    i--;
  strSize = (i - beginStr) + 1;
  result = (char*)malloc(strSize);
  result[strSize] = '\0';
  strncpy(result, (str + beginStr), strSize);
  return (result);
}

// strCopy()
// retourne une copie d'une chaine de caractres

char*	strCopy(const char* str)
{
  char*	result;

  result = (char*)malloc(strlen(str) + 1);
  strcpy(result, str);
  return (result);
}

// intToStr()
// retourne une chaine de caractres reprsentant l'entier pass en paramtre

char*	intToStr(int nbr)
{
  int	i;
  int	size;
  char*	tmp;
  char*	result;

  for (size = 0, i = nbr; i > 0; i /= 10, size++)
    ;
  //++size;
  tmp = (char*)malloc(size + 1);
  result = (char*)malloc(size + 1);
  for (i = 0; (i < (size - 1)) && (nbr >= 10); i++)
    {
      tmp[i] = '0' + (nbr % 10);
      nbr /= 10;
    }
  tmp[i] = '0' + nbr;
  result[size] = tmp[size] = '\0';
  for (i = 0; i < size; i++)
    {
      result[i] = tmp[size - (i + 1)];
    }
  free(tmp);
  return (result);
}
