/*
 ** tcp.h
 ** 
 ** Made by (amaury bouchard)
 ** Login   <boucha_a@epita.fr>
 ** 
 ** Started on  Sat Jul 10 10:32:53 1999 amaury bouchard
 ** Last update Thu Sep 30 11:09:06 1999 amaury bouchard
 */

// *************************** TCP LIB v2.0 ***********************************
// 
// librairie ayant pour but de creer facilement des connexions client-serveur
// basees sur le protocole TCP-IP.
//
// ajout par rapport a la v1.0 : gestion des exceptions
// par rapport a la 2.0 beta : utilisation des classes objet (cf Documentation)
//
// ****************************************************************************

#ifndef   __TCP_SERVER_H__
#define   __TCP_SERVER_H__

#include <netdb.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "trac.h"

// definition des etats possibles pour un fd (utilise ou non)
#define	FD_STATE_NOT_USED	0
#define	FD_STATE_USED		(1<<0)

// definition du type de connexion tcp (client ou server)
#define	SERVER_SOCK		1
#define	CLIENT_SOCK		2

// macros d'erreur
#define	TCP_ERROR_QUIT(x)	{perror(x);exit(1);}
#define	TCP_ERROR_CONTINUE(x)	{perror(x);continue;}

// definition du nombre d'exceptions gerees par la librairie
#define	NBR_EXCEP_MAX		64

// t_fonc_table
// structure permettant de controler chaque file descriptor ou exception
// il y a une structure de ce type qui est allouee pour chaque fd ou exception
// gere par la machine
//
// state	indique si le fd est utilise (c'est-a-dire doit etre gere par
//		la fonction qui boucle sur les fd) ou non
// fonc		pointeur sur la fonction associee au fd. cette fonction sera
//		executee si le fd recoit des donnees

typedef struct	s_fonc_table
{
   int		state;
   void		(*fonc)();
}		t_fonc_table;

// t_tcp_env
// structure contenant tous les renseignements sur l'environnement TCP
//
// port			numero de port sur lequel se fait la connexion
// server_name		nom du server (mis a 0 pour un client)
// sd_connect		contient le numero de la socket par laquelle se fait
//			la connexion
// loop_condition	la boucle sur les fd se fait tant que cet entier vaut 1
//			la boucle s'arrete si il vaut 0
// max_fd_connected	contient le numero du plus grand fd sur lequel
//			il faut boucler
// fdtablesize		taille de la table des fd geree par la machine
// max_excep		contient le plus grand numero d'exception sur laquelle
//			il faut boucler
// fd_table		tableau de t_fonc_table permettant d'associer des
//			fonctions aux fd
// excep_table		tableau de t_fonc_table permettant d'associer des
//			fonctions aux exceptions
// sread		c'est un fd_set permettant d'utiliser la fonction
// sexcep		c'est un fd_set permettant de controler les exceptions

class
{
  noextends;
  int		port;
  char		*serverName;
  int		sdConnect;
  int		*loopCondition;
  int		maxFdConnected;
  int		fdTableSize;
  int		maxExcep;
  t_fonc_table	*fdTable;
  t_fonc_table	*excepTable;
  fd_set	sread;
  fd_set	sexcep;
}		ClassTcp;

// Prototypes
// trac_tcp_init.c
ClassTcp*	newClassTcp(ClassTcp*,int,int,char*);
void		deleteClassTcp(ClassTcp*);
// trac_tcp_connection.c
void		tcpServerLoop(ClassTcp*,int*,void(*)());
void		tcpClientLoop(ClassTcp*,int*,void(*)());
// trac_tcp_fd_excep.c
void		tcpAddFd(ClassTcp*,int,void(*)());
void		tcpRemoveFd(ClassTcp*,int);
void		tcpAddExcep(ClassTcp*,int,void(*)());
void		tcpRemoveExcep(ClassTcp*,int);

#endif // __TCP_SERVER_H__
