#include "trac_tcp.h"

static void	tcpInitTables(ClassTcp*);
static void	tcpExecServerFonc(ClassTcp*);
static void	tcpExecClientFonc(ClassTcp*);

// tcpServerLoop() -- 26 lignes
// cree le server, et boucle sur les file descriptors

void	tcpServerLoop(ClassTcp* this,int* loopCond,void (*fonc)())
{
  this->maxFdConnected = this->sdConnect;
  this->fdTable[this->sdConnect].state = FD_STATE_USED;
  this->fdTable[this->sdConnect].fonc = fonc;
  this->loopCondition = loopCond;
  while (*(this->loopCondition))
    {
      tcpInitTables(this);
      if (select(this->maxFdConnected,&this->sread,0,&this->sexcep,0)<0)
	TCP_ERROR_CONTINUE("select");
      tcpExecServerFonc(this);
    }
}

// tcpClientLooop() -- 11 lignes
// cree le client, et boucle sur les file descriptors

void	tcpClientLoop(ClassTcp* this,int* loopCond,void (*fonc)())
{
  this->maxFdConnected = this->sdConnect;
  this->fdTable[this->sdConnect].state = FD_STATE_USED;
  this->fdTable[this->sdConnect].fonc = fonc;
  this->loopCondition = loopCond;
  while (*(this->loopCondition))
    {
      tcpInitTables(this);
      if (select(this->sdConnect+1,&this->sread,0,&this->sexcep,0)<0)
	TCP_ERROR_CONTINUE("select");
      tcpExecClientFonc(this);
    }
}

// tcpInitTables() -- 16 lignes
// pour chaque tour de boucle, reinitialise les fd_set de controle des fd et
// des exceptions

static void	tcpInitTables(ClassTcp* this)
{
   int		i;

   FD_ZERO(&this->sread);
   FD_ZERO(&this->sexcep);
   for (i = 0 ;i < this->fdTableSize;i++)
      if (this->fdTable[i].state & FD_STATE_USED)
      {
	 FD_SET(i,&this->sread);
	 this->maxFdConnected = i + 1;
      }
   for (i = 0;i < NBR_EXCEP_MAX;i++)
      if (this->excepTable[i].state & FD_STATE_USED)
      {
	 FD_SET(i,&this->sexcep);
	 this->maxExcep = i + 1;
      }
}

// tcpExecServerFonc() -- 28 lignes
// fonction executee lorsque des donnees ont ete recues sur un des fd ou qu'une
// exception a ete levee

static void	tcpExecServerFonc(ClassTcp* this)
{
  int			i;
  int			flag;
  int			newFd;
  struct sockaddr_in	name;
  int			size;
  
  for (i = flag = 0;i < this->fdTableSize;i++)
    if ((this->fdTable[i].state & FD_STATE_USED) && FD_ISSET(i,&this->sread))
      {
	if (i == this->sdConnect)
	  {
	    newFd = accept(i,(void*)&name,&size);
	    this->fdTable[newFd].state = FD_STATE_USED;
	    this->fdTable[newFd].fonc = this->fdTable[i].fonc;
	  }
	else
	  this->fdTable[i].fonc(this,i);
	flag = 1;
	break;
      }
  if (!flag)
    for (i = 0 ;i < NBR_EXCEP_MAX;i++)
      if ((this->excepTable[i].state & FD_STATE_USED)
	  && FD_ISSET(i,&this->sexcep))
	{
	  this->excepTable[i].fonc(this,i);
	  break;
	}
}

// exec_tcp_client_fonc() -- 18 lignes
// fonction appelee quand le client recoit des donnes sur un fd ou qu'une de
// ses exceptions a ete levee

static void	tcpExecClientFonc(ClassTcp* this)
{
  int		i;
  int		flag;
  
  for (i = flag = 0;i < this->fdTableSize;i++)
    if ((this->fdTable[i].state & FD_STATE_USED) && FD_ISSET(i,&this->sread))
      {
	this->fdTable[i].fonc(this,i);
	flag = 1;
	break;
      }
  if (!flag)
    for (i = 0 ;i < NBR_EXCEP_MAX;i++)
      if ((this->excepTable[i].state & FD_STATE_USED)
	  && FD_ISSET(i,&this->sexcep))
	{
	  this->excepTable[i].fonc(this,i);
	  break;
	}
}
