#include "trac_xml.h"
 
// =============================================================================
// prototypes privs
// =============================================================================
static int		_xmlVerifValidNewFileName(ClassXml* this,
						  const char* fileName);
static void		_xmlVerifBeginFile(ClassXml* this);
static void		_xmlCreateTag(ClassXml* this, const char* tag,
				      const char* value);
static int		_xmlGotoTag(ClassXml* this, const char* tag);
static char*		_xmlGotoTagPath(ClassXml* this, const char* tagPath,
					int gotoType);
static tDataSet*	_xmlGetTagData(ClassXml* this);
static tDataSet*	_xmlGetSousTagData(ClassXml* this, char* tagPath);

// =============================================================================
// fonctions publiques
// =============================================================================

// xmlCreate()
// cre un nouveau fichier XML

int	xmlCreate(ClassXml* this, const char* fileName)
{
  FILE*	file;
  int	result;

  try
    {
      _xmlVerifValidNewFileName(this, fileName);
      file = fopen(fileName, "w+");
      if (file == NULL)
	raise("Erreur  la cration du fichier XML");
      rewind(file);
      this->file = file;
      this->status = XML_STATUS_OPENED;
      fprintf(file, XML_NEW_FILE_STR);
      result = 1;      
    }
  except
    result = 0;
  endExcept;
  return (result);
}

// xmlOpen()
// ouvre un fichier XML dja existant
// retourne zro et lve une exception en cas d'erreur

int	xmlOpen(ClassXml* this, const char* fileName)
{
  FILE*	file;
  int	result;

  try
    {
      if (this->status != XML_STATUS_CLOSED)
	raise("Objet XML en cours d'utilisation");
      file = fopen(fileName, "r+");
      if (file == NULL)
	raise("Erreur  l'ouverture du fichier XML");
      rewind(file);
      this->file = file;
      _xmlVerifBeginFile(this);
      this->status = XML_STATUS_OPENED;
      result = 1;
    }
  except
    result = 0;
  endExcept;
  return (result);
}

// xmlClose()
// ferme l'utilisation d'un fichier XML par un objet ClassXml
// retourne zro et lve une exception en cas d'erreur

int	xmlClose(ClassXml* this)
{
  int	result;

  try
    {
      if (this->status != XML_STATUS_OPENED)
	raise("Impossible de fermer un objet XML non ouvert");
      if (fclose(this->file) != 0)
	raise("Erreur lors de la fermeture du fichier XML");
      this->status = XML_STATUS_CLOSED;
      result = 1;
    }
  except
    result = 0;
  endExcept;
  return (result);
}

// xmlGetData()
// retourne la chaine de caractres se trouvant dans le chemin de tags spcifi
// lve une exception si le chemin de tags n'existe pas

ClassList*	xmlGetData(ClassXml* this, const char* tagPath)
{
  char*		tag;
  tDataSet*	record;
  ClassList*	result;

  tag = _xmlGotoTagPath(this, tagPath, GOTO_TAGPATH_READ);
  result = newClassList(new(ClassList));
  record = _xmlGetTagData(this);
  record->name = strCopy(tag);
  listPutStart(result, record);
  while (_xmlGotoTag(this, tag))
    {
      record = _xmlGetTagData(this);
      record->name = strCopy(tag);
      listPutEnd(result, record);
    }
  return (result);
}

// xmlAddData
// ajoute une entre au fichier XML
int		xmlAddData(ClassXml* this, const char* tagPath,
			   ClassList* values)
{
  char*		tag;
  tDataSet*	data;

  if (tagPath)
    tag = _xmlGotoTagPath(this, tagPath, GOTO_TAGPATH_WRITE);
  while ((data = listGetNextElem(values)))
    {
      if (data->type == DATASET_ATOME)
	  _xmlCreateTag(this, data->name, data->value);
      else if (data->type == DATASET_COMPOSITE)
	{
	  _xmlCreateTag(this, data->name, NULL);
	  xmlAddData(this, NULL, data->recordSet);
	}
      else
	raise("Erreur de RecordSet");
    }
  return (1);
}

// deleteRecordSet()
// dtruit toute la mmoire alloue pour un RecordSet

void		deleteRecordSet(ClassList* recordSet)
{
  tDataSet*	pt;

  while ((pt = listGetNextElem(recordSet)))
    {
      free(pt->name);
      if (pt->value)
	free(pt->value);
      if (pt->type == DATASET_COMPOSITE && pt->recordSet)
	deleteRecordSet(pt->recordSet);
      free(pt);
    }
  delete(recordSet);
}

// =============================================================================
// constructeur / destructeur
// =============================================================================

// newClassXml()
// constructeur de l'objet ClassXml

ClassXml*	newClassXml(ClassXml* this)
{
  initParentClass(this, deleteClassXml);
  return (this);
}

// deleteClassXml()
// destructeur de l'objet ClassXml

void	deleteClassXml(ClassXml* this)
{
  if (this->status == XML_STATUS_OPENED)
    xmlClose(this);
}

// =============================================================================
// fonctions prives
// =============================================================================

// _verifValidNewFileName()
// vrifie qu'un nom de fichier est valide et qu'il n'est pas dja utilis
// lve une exception en cas d'erreur

static int	_xmlVerifValidNewFileName(ClassXml* this, const char* fileName)
{
  FILE*		file;
  char*		tmp;
  char*		usedFileName;
  int		fileNameSize;

  if (this->status != XML_STATUS_CLOSED)
    raise("Objet XML en cours d'utilisation");
  file = fopen(fileName, "r+");
  if (file != NULL)
    {
      fclose(file);
      raise("Impossible de crer un fichier XML existant");
    }
  tmp = lowerCase(fileName);
  usedFileName = trim(tmp);
  free(tmp);
  tmp = strstr(usedFileName, SUFFIXE);
  if (!(fileNameSize = strlen(usedFileName)) || !tmp
      || tmp != (usedFileName + fileNameSize - strlen(SUFFIXE)))
    raise("Nom de fichier incorrect");
  return (1);
}

// _xmlVerifBeginFile()
// lit le dbut du fichier XML qui vient d'tre ouvert afin de s'assurer que
// c'est un fichier XML valide, avec le tag <DATA> de pllus haut niveau

static void	_xmlVerifBeginFile(ClassXml* this)
{
  int		insideTag, tagXml;
  int		caracCompare;
  char*		dataTag = DATA;
  char*		xmlTag = XML;
  char		c;

  insideTag = tagXml = caracCompare = 0;
  rewind(this->file);
  while ((c = fgetc(this->file)) != EOF)
    {
      if (c != SPACE && c != TAB && c != '\n')
	{
	  if (c == INF)
	    {
	      if (insideTag)
		raise("Erreur de syntaxe XML");
	      else
		insideTag = 1;
	    }
	  else if (c == SUP)
	    {
	      if (!insideTag || tagXml != 2)
		raise("Erreur de syntaxe XML");
	      else if (!dataTag[caracCompare])
		return ;
	      insideTag = 0;
	    }
	  else if (c == INTERROG)
	    {
	      if (tagXml == 1 && xmlTag[caracCompare])
		raise("Erreur de syntaxe XML");
	      else if (tagXml == 1 && !xmlTag[caracCompare])
		{
		  tagXml = 2;
		  caracCompare = 0;
		}
	      else
		tagXml = 1;
	    }
	  else if (c == SLASH)
	    raise("Erreur de syntaxe XML");
	  else
	    {
	      if (tagXml == 1 && xmlTag[caracCompare])
		{
		  if (c != xmlTag[caracCompare])
		    raise("Erreur de syntaxe XML");
		  else
		    ++caracCompare;
		}
	      else if (tagXml == 2)
		{
		  if (!insideTag)
		    raise("Erreur de syntaxe XML");
		  else if (c != dataTag[caracCompare])
		    raise("Erreur de syntaxe XML");
		  else
		    ++caracCompare;
		}
	    }
	}
    }
}

// _xmlCreateTag()
// ajoute un tag ouvrant et son tag fermant associ, puis place le file
// descriptor entre les deux s'il n'y avait pas de texte  y crire, car sinon
// place le file descriptor aprs le tag fermant

static void	_xmlCreateTag(ClassXml* this, const char* tag, const char* value)
{
  FILE*		tmpFile;
  fpos_t*	pos;
  char*		buffer;
  char*		pt;

  buffer = (char*)malloc(BUFF_SIZE);
  tmpFile = tmpfile();
  pos = new(fpos_t);
  fgetpos(this->file, pos);
  while ((pt = fgets(buffer, BUFF_SIZE, this->file)))
    fputs(pt, tmpFile);
  fsetpos(this->file, pos);
  fputc('\t', this->file);
  fputc(INF, this->file);
  fputs(tag, this->file);
  fputc(SUP, this->file);
  if (value)
    fputs(value, this->file);
  else
    fgetpos(this->file, pos);
  fputc(INF, this->file);
  fputc(SLASH, this->file);
  fputs(tag, this->file);
  fputc(SUP, this->file);
  fputc('\n', this->file);
  if (value)
    fgetpos(this->file, pos);
  rewind(tmpFile);
  while ((pt = fgets(buffer, BUFF_SIZE, tmpFile)))
    fputs(pt, this->file);
  fsetpos(this->file, pos);
  free(pos);
  fclose(tmpFile);
  free(buffer);
}

// _xmlGotoTag()
// positionne le pointeur de l'objet ClassXml  l'intrieur du tag demand
// si le tag n'existe pas, le pointeur est positionn juste avant la fermeture
// du tag de niveau suprieur, et retourne 0

static int	_xmlGotoTag(ClassXml* this, const char* tag)
{
  int		nbrSousTag, insideTag, closeTag;
  int		caracCompare, badTag;
  char		c;

  nbrSousTag = insideTag = closeTag = caracCompare = badTag = 0;
  while ((c = fgetc(this->file)) != EOF)
    {
      if (c == INF)
	{
	  if (insideTag)
	    raise("Erreur de syntaxe XML");
	  insideTag = 1;
	  closeTag = caracCompare = badTag = 0;
	}
      else if (c == SUP)
	{
	  insideTag = 0;
	  if (!closeTag)
	    if (!badTag && !tag[caracCompare])
	      return (1);
	    else
	      ++nbrSousTag;
	  else
	    --nbrSousTag;
	}
      else if (c == SLASH)
	{
	  if (closeTag)
	    raise("Erreur de syntaxe XML");
	  if (nbrSousTag == 0)
	    {
	      fseek(this->file, -2, SEEK_CUR);
	      return (0);
	    }
	  closeTag = 1;
	}
      else
	{
	  if (!badTag && insideTag)
	    {
	      if (!tag[caracCompare] || tag[caracCompare] != c)
		badTag = 1;
	      ++caracCompare;
	    }
	}
    }
  raise("Erreur de syntaxe XML");
  return (0);
}

// _xmlGotoTagPath()
// positionne le file descriptor  l'intrieur du chemin de tags demand et
// retourne le nom du dernier tag du chemin
// lve une exception en cas d'erreur

static char*	_xmlGotoTagPath(ClassXml* this, const char* tagPath,
				int gotoType)
{
  char*		realTagPath;
  char*		tag;
  char*		tag2;
  char*		tmp;

  realTagPath = strCopy(tagPath);
  if (realTagPath[0] == SLASH)
    _xmlVerifBeginFile(this);
  if ((tag = strtok(realTagPath, STR_SLASH)) != NULL)
    {
      tmp = tag;
      tag2 = strtok(NULL, STR_SLASH);
      if (gotoType == GOTO_TAGPATH_READ && !_xmlGotoTag(this, tag))
	raise("Balise XML inexistante");
      else if (gotoType == GOTO_TAGPATH_WRITE &&
	       (!_xmlGotoTag(this, tag) || !tag2))
	_xmlCreateTag(this, tag, NULL);
    }
  else
    tmp = realTagPath;
  while (tag2)
    {
      tag = tmp = tag2;
      tag2 = strtok(NULL, STR_SLASH);
      if (gotoType == GOTO_TAGPATH_READ && !_xmlGotoTag(this, tag))
	raise("Balise XML inexistante");
      else if (gotoType == GOTO_TAGPATH_WRITE &&
	       (!_xmlGotoTag(this, tag) || !tag2))
	_xmlCreateTag(this, tag, NULL);      
    }
  tmp = (tag == NULL) ? tmp : tag;
  return (tmp);
}

// _xmlGetTagData()
// retourne la chaine de caractres trouve  l'intrieur du tag o est
// positionn le file descriptor

static tDataSet*	_xmlGetTagData(ClassXml* this)
{
  char*			tmp;
  ClassString*		str;
  ClassString*		nameStr;
  ClassString*		trimStr;
  char			buffer[BUFF_SIZE];
  int			buffIndex;
  char			nameBuff[BUFF_SIZE];
  int			nameIndex;
  int			findSlash;
  int			endFlag;
  char			c;
  int			listeFlag;
  tDataSet*		record;
  tDataSet*		result;

  result = new(tDataSet);
  result->type = DATASET_ATOME;
  str = newClassString(new(ClassString), strCopy(""));
  for (buffIndex = endFlag = listeFlag = 0;
       (c = fgetc(this->file)) != EOF && !endFlag; buffIndex++)
    {
      if (c == INF)
	{
	  nameStr = newClassString(new(ClassString), strCopy(""));
	  for (nameIndex = 0, findSlash = 0;
	       (c = fgetc(this->file)) != EOF && c != SUP; nameIndex++)
	    {
	      if (c == SLASH || findSlash)
		findSlash = 1;
	      else
		{
		  if (nameIndex == (BUFF_SIZE - 1))
		    {
		      nameBuff[nameIndex] = '\0';
		      stringConcat(nameStr, nameBuff);
		      nameIndex = 0;
		    }
		  nameBuff[nameIndex] = c;
		}
	    }
	  if (c == SUP && !findSlash)
	    {
	      if (nameIndex)
		{
		  nameBuff[nameIndex] = '\0';
		  stringConcat(nameStr, nameBuff);
		}
	      fseek(this->file, -(strlen(nameStr->str) + 2), SEEK_CUR);
	      if (!listeFlag)
		{
		  result->recordSet = newClassList(new(ClassList));
		  listeFlag = 1;
		}
	      record = _xmlGetSousTagData(this, nameStr->str);
	      listPutEnd(result->recordSet, record);
	      result->type = DATASET_COMPOSITE;
	    }
	  else if (c == SUP && findSlash)
	    endFlag = 1;
	  delete(nameStr);
	}
      else
	{
	  if (buffIndex == (BUFF_SIZE - 1))
	    {
	      buffer[buffIndex] = '\0';
	      stringConcat(str, buffer);
	      buffIndex = 0;
	    }
	  buffer[buffIndex] = c;
	}
    }
  if (c == EOF)
    raise("Erreur de syntaxe XML");
  if (buffIndex > 1)
    {
      buffer[buffIndex - 1] = '\0';
      stringConcat(str, buffer);
    }
  if (result->type == DATASET_ATOME)
    {
      trimStr = stringTrim(str);
      tmp = strCopy(trimStr->str);
      delete(trimStr);
      result->value = tmp;
    }
  delete(str);
  return (result);
}

// xmlGetSousTagData()
// retourne la chaine de caractres se trouvant dans le chemin de tags spcifi
// lve une exception si le chemin de tags n'existe pas

static tDataSet*	_xmlGetSousTagData(ClassXml* this, char* tagPath)
{
  char*			realTagPath;
  char*			tag;
  char*			tmp;
  tDataSet*		record;

  realTagPath = strCopy(tagPath);
  if (realTagPath[0] == SLASH)
    _xmlVerifBeginFile(this);
  if ((tag = strtok(realTagPath, STR_SLASH)) != NULL)
    {
      tmp = tag;
      if (!_xmlGotoTag(this, tag))
	raise("Balise XML inexistante");
    }
  while ((tag = strtok(NULL, STR_SLASH)) != NULL)
    {
      tmp = tag;
      if (!_xmlGotoTag(this, tag))
	raise("Balise XML inexistante");
    }
  tmp =  (tag == NULL) ? tmp : tag;
  record = _xmlGetTagData(this);
  record->name = strCopy(tmp);
  return (record);
}
